@@ -33,13 +33,22 @@ from .constants import (
     PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY,
     PARAMETER_CLIENT_SESSION_KEEP_ALIVE,
     PARAMETER_CLIENT_TELEMETRY_ENABLED,
+    PARAMETER_CLIENT_TELEMETRY_OOB_ENABLED,
     PARAMETER_TIMEZONE,
     PARAMETER_SERVICE_NAME,
     PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL,
     PARAMETER_CLIENT_PREFETCH_THREADS,
+    PARAMETER_CLIENT_VALIDATE_DEFAULT_PARAMETERS,
     OCSPMode
 )
 from .cursor import SnowflakeCursor, LOG_MAX_QUERY_LENGTH
+from .description import (
+    SNOWFLAKE_CONNECTOR_VERSION,
+    PYTHON_VERSION,
+    PLATFORM,
+    CLIENT_NAME,
+    CLIENT_VERSION
+)
 from .errorcode import (ER_CONNECTION_IS_CLOSED,
                         ER_NO_ACCOUNT_NAME, ER_OLD_PYTHON, ER_NO_USER,
                         ER_NO_PASSWORD, ER_INVALID_VALUE,
@@ -48,11 +57,6 @@ from .errorcode import (ER_CONNECTION_IS_CLOSED,
 from .errors import (Error, ProgrammingError, InterfaceError,
                      DatabaseError)
 from .network import (
-    SNOWFLAKE_CONNECTOR_VERSION,
-    PYTHON_VERSION,
-    PLATFORM,
-    CLIENT_NAME,
-    CLIENT_VERSION,
     DEFAULT_AUTHENTICATOR,
     EXTERNAL_BROWSER_AUTHENTICATOR,
     KEY_PAIR_AUTHENTICATOR,
@@ -63,6 +67,7 @@ from .network import (
 from .sqlstate import (SQLSTATE_CONNECTION_NOT_EXISTS,
                        SQLSTATE_FEATURE_NOT_SUPPORTED)
 from .telemetry import (TelemetryClient)
+from .telemetry_oob import TelemetryService
 from .time_util import (
     DEFAULT_MASTER_VALIDITY_IN_SECONDS,
     HeartBeatTimer, get_time_millis)
@@ -160,6 +165,8 @@ class SnowflakeConnection(object):
     connect(..) to get the object.
     """
 
+    OCSP_ENV_LOCK = Lock()
+
     def __init__(self, **kwargs):
         self._lock_sequence_counter = Lock()
         self.sequence_counter = 0
@@ -473,6 +480,7 @@ class SnowflakeConnection(object):
         logger.debug(u'connect')
         if len(kwargs) > 0:
             self.__config(**kwargs)
+            TelemetryService.get_instance().update_context(kwargs)
 
         self.__set_error_attributes()
         self.__open_connection()
@@ -603,30 +611,39 @@ class SnowflakeConnection(object):
 
     @staticmethod
     def setup_ocsp_privatelink(app, hostname):
-        if app == APPLICATION_SNOWSQL:
+        SnowflakeConnection.OCSP_ENV_LOCK.acquire()
+        if app is APPLICATION_SNOWSQL:
             ocsp_cache_server = u'http://ocsp{}/ocsp_response_cache.json'.format(
                 hostname[hostname.index('.'):])
+            '''
+             Check if user has configured a custom OCSP Cache Server URL
+             '''
+            if 'SF_OCSP_RESPONSE_CACHE_SERVER_URL' not in os.environ:
+                os.environ[
+                    'SF_OCSP_RESPONSE_CACHE_SERVER_URL'] = ocsp_cache_server
+            else:
+                if "ocsp_response_cache" not in os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL']:
+                    if not os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL']. \
+                            startswith("http://"):
+                        ocsp_cache_server = "http://{0}/{1}".format(
+                            os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL'],
+                            "ocsp_response_cache.json")
+                    else:
+                        ocsp_cache_server = "{0}/{1}".format(
+                            os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL'],
+                            "ocsp_response_cache.json")
+                else:
+                    ocsp_cache_server = os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL']
+
+                os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL'] = ocsp_cache_server
         else:
             ocsp_cache_server = \
                 u'http://ocsp.{}/ocsp_response_cache.json'.format(
                     hostname)
-        if 'SF_OCSP_RESPONSE_CACHE_SERVER_URL' not in os.environ:
-            os.environ[
-                'SF_OCSP_RESPONSE_CACHE_SERVER_URL'] = ocsp_cache_server
-        else:
-            if not os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL']. \
-                    startswith("http://"):
-                ocsp_cache_server = "http://{0}/{1}".format(
-                    os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL'],
-                    "ocsp_response_cache.json")
-            else:
-                ocsp_cache_server = "{0}/{1}".format(
-                    os.environ['SF_OCSP_RESPONSE_CACHE_SERVER_URL'],
-                    "ocsp_response_cache.json")
-
             os.environ[
                 'SF_OCSP_RESPONSE_CACHE_SERVER_URL'] = ocsp_cache_server
         logger.debug(u"OCSP Cache Server is updated: %s", ocsp_cache_server)
+        SnowflakeConnection.OCSP_ENV_LOCK.release()
 
     def __open_connection(self):
         u"""
@@ -683,6 +700,10 @@ class SnowflakeConnection(object):
         if self._timezone is not None:
             self._session_parameters[PARAMETER_TIMEZONE] = self._timezone
 
+        if self._validate_default_parameters:
+            # Snowflake will validate the requested database, schema, and warehouse
+            self._session_parameters[PARAMETER_CLIENT_VALIDATE_DEFAULT_PARAMETERS] = True
+
         if self.client_session_keep_alive:
             self._session_parameters[PARAMETER_CLIENT_SESSION_KEEP_ALIVE] = True
 
@@ -1141,6 +1162,11 @@ class SnowflakeConnection(object):
             session_parameters[name] = value
             if PARAMETER_CLIENT_TELEMETRY_ENABLED == name:
                 self.telemetry_enabled = value
+            elif PARAMETER_CLIENT_TELEMETRY_OOB_ENABLED == name:
+                if value:
+                    TelemetryService.get_instance().enable()
+                else:
+                    TelemetryService.get_instance().disable()
             elif PARAMETER_CLIENT_SESSION_KEEP_ALIVE == name:
                 self.client_session_keep_alive = value
             elif PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY == \
