@@ -9,9 +9,9 @@ import gzip
 import itertools
 import json
 import logging
-import platform
 import sys
 import time
+import traceback
 import uuid
 from io import BytesIO
 from threading import Lock
@@ -37,7 +37,17 @@ from .constants import (
     HTTP_HEADER_CONTENT_TYPE,
     HTTP_HEADER_ACCEPT,
     HTTP_HEADER_USER_AGENT,
-    HTTP_HEADER_SERVICE_NAME,
+    HTTP_HEADER_SERVICE_NAME
+)
+from .description import (
+    SNOWFLAKE_CONNECTOR_VERSION,
+    PYTHON_VERSION,
+    OPERATING_SYSTEM,
+    PLATFORM,
+    IMPLEMENTATION,
+    COMPILER,
+    CLIENT_NAME,
+    CLIENT_VERSION
 )
 from .errorcode import (ER_FAILED_TO_CONNECT_TO_DB, ER_CONNECTION_IS_CLOSED,
                         ER_FAILED_TO_REQUEST, ER_FAILED_TO_RENEW_SESSION)
@@ -48,13 +58,14 @@ from .errors import (Error, OperationalError, DatabaseError, ProgrammingError,
                      OtherHTTPRetryableError)
 from .sqlstate import (SQLSTATE_CONNECTION_NOT_EXISTS,
                        SQLSTATE_CONNECTION_WAS_NOT_ESTABLISHED,
-                       SQLSTATE_CONNECTION_REJECTED)
+                       SQLSTATE_CONNECTION_REJECTED,
+                       SQLSTATE_IO_ERROR)
+from .telemetry_oob import TelemetryService
 from .time_util import (
     DecorrelateJitterBackoff,
     DEFAULT_MASTER_VALIDITY_IN_SECONDS
 )
 from .tool.probe_connection import probe_connection
-from .version import VERSION
 
 if PY2:
     from pyasn1.error import PyAsn1Error
@@ -100,15 +111,15 @@ REQUEST_ID = u'requestId'
 REQUEST_GUID = u'request_guid'
 SNOWFLAKE_HOST_SUFFIX = u'.snowflakecomputing.com'
 
-SNOWFLAKE_CONNECTOR_VERSION = u'.'.join(TO_UNICODE(v) for v in VERSION[0:3])
-PYTHON_VERSION = u'.'.join(TO_UNICODE(v) for v in sys.version_info[:3])
-OPERATING_SYSTEM = platform.system()
-PLATFORM = platform.platform()
-IMPLEMENTATION = platform.python_implementation()
-COMPILER = platform.python_compiler()
+SNOWFLAKE_CONNECTOR_VERSION = SNOWFLAKE_CONNECTOR_VERSION
+PYTHON_VERSION = PYTHON_VERSION
+OPERATING_SYSTEM = OPERATING_SYSTEM
+PLATFORM = PLATFORM
+IMPLEMENTATION = IMPLEMENTATION
+COMPILER = COMPILER
 
-CLIENT_NAME = u"PythonConnector"  # don't change!
-CLIENT_VERSION = u'.'.join([TO_UNICODE(v) for v in VERSION[:3]])
+CLIENT_NAME = CLIENT_NAME  # don't change!
+CLIENT_VERSION = CLIENT_VERSION
 PYTHON_CONNECTOR_USER_AGENT = \
     u'{name}/{version}/{python_implementation}/{python_version}/{platform}'.format(
         name=CLIENT_NAME,
@@ -554,6 +565,7 @@ class SnowflakeRestful(object):
 
         class RetryCtx(object):
             def __init__(self, timeout, _include_retry_params=False):
+                self.total_timeout = timeout
                 self.timeout = timeout
                 self.cnt = 0
                 self.sleeping_time = 1
@@ -632,6 +644,19 @@ class SnowflakeRestful(object):
                 method, full_url, headers, data, conn)
             return {}
         except RetryRequest as e:
+            if retry_ctx.cnt == TelemetryService.get_instance().num_of_retry_to_trigger_telemetry:
+                _, _, stack_trace = sys.exc_info()
+                TelemetryService.get_instance().log_http_request(
+                    "HttpRequestRetry%dTimes" % retry_ctx.cnt,
+                    full_url,
+                    method,
+                    SQLSTATE_IO_ERROR,
+                    ER_FAILED_TO_REQUEST,
+                    retry_timeout=retry_ctx.total_timeout,
+                    retry_count=retry_ctx.cnt,
+                    exception=str(e),
+                    stack_trace=traceback.format_exc()
+                )
             if no_retry:
                 return {}
             cause = e.args[0]
@@ -639,6 +664,18 @@ class SnowflakeRestful(object):
                 retry_ctx.timeout -= int(time.time() - start_request_thread)
                 if retry_ctx.timeout <= 0:
                     logger.error(cause, exc_info=True)
+                    _, _, stack_trace = sys.exc_info()
+                    TelemetryService.get_instance().log_http_request(
+                        "HttpRequestRetryTimeout",
+                        full_url,
+                        method,
+                        SQLSTATE_IO_ERROR,
+                        ER_FAILED_TO_REQUEST,
+                        retry_timeout=retry_ctx.total_timeout,
+                        retry_count=retry_ctx.cnt,
+                        exception=str(e),
+                        stack_trace=traceback.format_exc()
+                    )
                     if isinstance(cause, Error):
                         Error.errorhandler_wrapper(conn, None, cause)
                     else:
@@ -790,6 +827,14 @@ class SnowflakeRestful(object):
                     )
                     return None  # required for tests
                 else:
+                    TelemetryService.get_instance().log_http_request(
+                        "HttpError%s" % str(raw_ret.status_code),
+                        full_url,
+                        method,
+                        SQLSTATE_CONNECTION_WAS_NOT_ESTABLISHED,
+                        ER_FAILED_TO_REQUEST,
+                        response=raw_ret
+                    )
                     Error.errorhandler_wrapper(
                         self._connection, None, InterfaceError,
                         {
@@ -831,6 +876,16 @@ class SnowflakeRestful(object):
                     "Ignore the following error stack: %s", err,
                     exc_info=True)
                 raise RetryRequest(err)
+            _, _, stack_trace = sys.exc_info()
+            TelemetryService.get_instance().log_http_request(
+                "HttpException%s" % str(err),
+                full_url,
+                method,
+                SQLSTATE_CONNECTION_WAS_NOT_ESTABLISHED,
+                ER_FAILED_TO_REQUEST,
+                exception=err,
+                stack_trace=traceback.format_exc()
+            )
             raise err
 
     def make_requests_session(self):
